// Copyright 2019 The Kubernetes Authors.
// SPDX-License-Identifier: Apache-2.0

// Package konfig provides configuration methods and constants
// for the kustomize API, e.g. the set of file names to look for
// to identify a kustomization root.
package konfig
