// Copyright 2022 The Kubernetes Authors.
// SPDX-License-Identifier: Apache-2.0

// Package nameref contains a kio.Filter implementation of the kustomize
// name reference transformer.
package nameref
