// Copyright 2020 The Kubernetes Authors.
// SPDX-License-Identifier: Apache-2.0

// Package namespace contains a kio.Filter implementation of the kustomize
// patchjson6902 transformer
package patchjson6902
