// Copyright The Helm Authors.
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//go:build darwin

package helmpath

import (
	"path/filepath"

	"k8s.io/client-go/util/homedir"
)

func dataHome() string {
	return filepath.Join(homedir.HomeDir(), "Library")
}

func configHome() string {
	return filepath.Join(homedir.HomeDir(), "Library", "Preferences")
}

func cacheHome() string {
	return filepath.Join(homedir.HomeDir(), "Library", "Caches")
}
