//go:build kubernetes_protomessage_one_more_release
// +build kubernetes_protomessage_one_more_release

/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by go-to-protobuf. DO NOT EDIT.

package v1

func (*AWSElasticBlockStoreVolumeSource) ProtoMessage() {}

func (*Affinity) ProtoMessage() {}

func (*AppArmorProfile) ProtoMessage() {}

func (*AttachedVolume) ProtoMessage() {}

func (*AvoidPods) ProtoMessage() {}

func (*AzureDiskVolumeSource) ProtoMessage() {}

func (*AzureFilePersistentVolumeSource) ProtoMessage() {}

func (*AzureFileVolumeSource) ProtoMessage() {}

func (*Binding) ProtoMessage() {}

func (*CSIPersistentVolumeSource) ProtoMessage() {}

func (*CSIVolumeSource) ProtoMessage() {}

func (*Capabilities) ProtoMessage() {}

func (*CephFSPersistentVolumeSource) ProtoMessage() {}

func (*CephFSVolumeSource) ProtoMessage() {}

func (*CinderPersistentVolumeSource) ProtoMessage() {}

func (*CinderVolumeSource) ProtoMessage() {}

func (*ClientIPConfig) ProtoMessage() {}

func (*ClusterTrustBundleProjection) ProtoMessage() {}

func (*ComponentCondition) ProtoMessage() {}

func (*ComponentStatus) ProtoMessage() {}

func (*ComponentStatusList) ProtoMessage() {}

func (*ConfigMap) ProtoMessage() {}

func (*ConfigMapEnvSource) ProtoMessage() {}

func (*ConfigMapKeySelector) ProtoMessage() {}

func (*ConfigMapList) ProtoMessage() {}

func (*ConfigMapNodeConfigSource) ProtoMessage() {}

func (*ConfigMapProjection) ProtoMessage() {}

func (*ConfigMapVolumeSource) ProtoMessage() {}

func (*Container) ProtoMessage() {}

func (*ContainerExtendedResourceRequest) ProtoMessage() {}

func (*ContainerImage) ProtoMessage() {}

func (*ContainerPort) ProtoMessage() {}

func (*ContainerResizePolicy) ProtoMessage() {}

func (*ContainerRestartRule) ProtoMessage() {}

func (*ContainerRestartRuleOnExitCodes) ProtoMessage() {}

func (*ContainerState) ProtoMessage() {}

func (*ContainerStateRunning) ProtoMessage() {}

func (*ContainerStateTerminated) ProtoMessage() {}

func (*ContainerStateWaiting) ProtoMessage() {}

func (*ContainerStatus) ProtoMessage() {}

func (*ContainerUser) ProtoMessage() {}

func (*DaemonEndpoint) ProtoMessage() {}

func (*DownwardAPIProjection) ProtoMessage() {}

func (*DownwardAPIVolumeFile) ProtoMessage() {}

func (*DownwardAPIVolumeSource) ProtoMessage() {}

func (*EmptyDirVolumeSource) ProtoMessage() {}

func (*EndpointAddress) ProtoMessage() {}

func (*EndpointPort) ProtoMessage() {}

func (*EndpointSubset) ProtoMessage() {}

func (*Endpoints) ProtoMessage() {}

func (*EndpointsList) ProtoMessage() {}

func (*EnvFromSource) ProtoMessage() {}

func (*EnvVar) ProtoMessage() {}

func (*EnvVarSource) ProtoMessage() {}

func (*EphemeralContainer) ProtoMessage() {}

func (*EphemeralContainerCommon) ProtoMessage() {}

func (*EphemeralVolumeSource) ProtoMessage() {}

func (*Event) ProtoMessage() {}

func (*EventList) ProtoMessage() {}

func (*EventSeries) ProtoMessage() {}

func (*EventSource) ProtoMessage() {}

func (*ExecAction) ProtoMessage() {}

func (*FCVolumeSource) ProtoMessage() {}

func (*FileKeySelector) ProtoMessage() {}

func (*FlexPersistentVolumeSource) ProtoMessage() {}

func (*FlexVolumeSource) ProtoMessage() {}

func (*FlockerVolumeSource) ProtoMessage() {}

func (*GCEPersistentDiskVolumeSource) ProtoMessage() {}

func (*GRPCAction) ProtoMessage() {}

func (*GitRepoVolumeSource) ProtoMessage() {}

func (*GlusterfsPersistentVolumeSource) ProtoMessage() {}

func (*GlusterfsVolumeSource) ProtoMessage() {}

func (*HTTPGetAction) ProtoMessage() {}

func (*HTTPHeader) ProtoMessage() {}

func (*HostAlias) ProtoMessage() {}

func (*HostIP) ProtoMessage() {}

func (*HostPathVolumeSource) ProtoMessage() {}

func (*ISCSIPersistentVolumeSource) ProtoMessage() {}

func (*ISCSIVolumeSource) ProtoMessage() {}

func (*ImageVolumeSource) ProtoMessage() {}

func (*KeyToPath) ProtoMessage() {}

func (*Lifecycle) ProtoMessage() {}

func (*LifecycleHandler) ProtoMessage() {}

func (*LimitRange) ProtoMessage() {}

func (*LimitRangeItem) ProtoMessage() {}

func (*LimitRangeList) ProtoMessage() {}

func (*LimitRangeSpec) ProtoMessage() {}

func (*LinuxContainerUser) ProtoMessage() {}

func (*List) ProtoMessage() {}

func (*LoadBalancerIngress) ProtoMessage() {}

func (*LoadBalancerStatus) ProtoMessage() {}

func (*LocalObjectReference) ProtoMessage() {}

func (*LocalVolumeSource) ProtoMessage() {}

func (*ModifyVolumeStatus) ProtoMessage() {}

func (*NFSVolumeSource) ProtoMessage() {}

func (*Namespace) ProtoMessage() {}

func (*NamespaceCondition) ProtoMessage() {}

func (*NamespaceList) ProtoMessage() {}

func (*NamespaceSpec) ProtoMessage() {}

func (*NamespaceStatus) ProtoMessage() {}

func (*Node) ProtoMessage() {}

func (*NodeAddress) ProtoMessage() {}

func (*NodeAffinity) ProtoMessage() {}

func (*NodeCondition) ProtoMessage() {}

func (*NodeConfigSource) ProtoMessage() {}

func (*NodeConfigStatus) ProtoMessage() {}

func (*NodeDaemonEndpoints) ProtoMessage() {}

func (*NodeFeatures) ProtoMessage() {}

func (*NodeList) ProtoMessage() {}

func (*NodeProxyOptions) ProtoMessage() {}

func (*NodeRuntimeHandler) ProtoMessage() {}

func (*NodeRuntimeHandlerFeatures) ProtoMessage() {}

func (*NodeSelector) ProtoMessage() {}

func (*NodeSelectorRequirement) ProtoMessage() {}

func (*NodeSelectorTerm) ProtoMessage() {}

func (*NodeSpec) ProtoMessage() {}

func (*NodeStatus) ProtoMessage() {}

func (*NodeSwapStatus) ProtoMessage() {}

func (*NodeSystemInfo) ProtoMessage() {}

func (*ObjectFieldSelector) ProtoMessage() {}

func (*ObjectReference) ProtoMessage() {}

func (*PersistentVolume) ProtoMessage() {}

func (*PersistentVolumeClaim) ProtoMessage() {}

func (*PersistentVolumeClaimCondition) ProtoMessage() {}

func (*PersistentVolumeClaimList) ProtoMessage() {}

func (*PersistentVolumeClaimSpec) ProtoMessage() {}

func (*PersistentVolumeClaimStatus) ProtoMessage() {}

func (*PersistentVolumeClaimTemplate) ProtoMessage() {}

func (*PersistentVolumeClaimVolumeSource) ProtoMessage() {}

func (*PersistentVolumeList) ProtoMessage() {}

func (*PersistentVolumeSource) ProtoMessage() {}

func (*PersistentVolumeSpec) ProtoMessage() {}

func (*PersistentVolumeStatus) ProtoMessage() {}

func (*PhotonPersistentDiskVolumeSource) ProtoMessage() {}

func (*Pod) ProtoMessage() {}

func (*PodAffinity) ProtoMessage() {}

func (*PodAffinityTerm) ProtoMessage() {}

func (*PodAntiAffinity) ProtoMessage() {}

func (*PodAttachOptions) ProtoMessage() {}

func (*PodCertificateProjection) ProtoMessage() {}

func (*PodCondition) ProtoMessage() {}

func (*PodDNSConfig) ProtoMessage() {}

func (*PodDNSConfigOption) ProtoMessage() {}

func (*PodExecOptions) ProtoMessage() {}

func (*PodExtendedResourceClaimStatus) ProtoMessage() {}

func (*PodIP) ProtoMessage() {}

func (*PodList) ProtoMessage() {}

func (*PodLogOptions) ProtoMessage() {}

func (*PodOS) ProtoMessage() {}

func (*PodPortForwardOptions) ProtoMessage() {}

func (*PodProxyOptions) ProtoMessage() {}

func (*PodReadinessGate) ProtoMessage() {}

func (*PodResourceClaim) ProtoMessage() {}

func (*PodResourceClaimStatus) ProtoMessage() {}

func (*PodSchedulingGate) ProtoMessage() {}

func (*PodSecurityContext) ProtoMessage() {}

func (*PodSignature) ProtoMessage() {}

func (*PodSpec) ProtoMessage() {}

func (*PodStatus) ProtoMessage() {}

func (*PodStatusResult) ProtoMessage() {}

func (*PodTemplate) ProtoMessage() {}

func (*PodTemplateList) ProtoMessage() {}

func (*PodTemplateSpec) ProtoMessage() {}

func (*PortStatus) ProtoMessage() {}

func (*PortworxVolumeSource) ProtoMessage() {}

func (*Preconditions) ProtoMessage() {}

func (*PreferAvoidPodsEntry) ProtoMessage() {}

func (*PreferredSchedulingTerm) ProtoMessage() {}

func (*Probe) ProtoMessage() {}

func (*ProbeHandler) ProtoMessage() {}

func (*ProjectedVolumeSource) ProtoMessage() {}

func (*QuobyteVolumeSource) ProtoMessage() {}

func (*RBDPersistentVolumeSource) ProtoMessage() {}

func (*RBDVolumeSource) ProtoMessage() {}

func (*RangeAllocation) ProtoMessage() {}

func (*ReplicationController) ProtoMessage() {}

func (*ReplicationControllerCondition) ProtoMessage() {}

func (*ReplicationControllerList) ProtoMessage() {}

func (*ReplicationControllerSpec) ProtoMessage() {}

func (*ReplicationControllerStatus) ProtoMessage() {}

func (*ResourceClaim) ProtoMessage() {}

func (*ResourceFieldSelector) ProtoMessage() {}

func (*ResourceHealth) ProtoMessage() {}

func (*ResourceQuota) ProtoMessage() {}

func (*ResourceQuotaList) ProtoMessage() {}

func (*ResourceQuotaSpec) ProtoMessage() {}

func (*ResourceQuotaStatus) ProtoMessage() {}

func (*ResourceRequirements) ProtoMessage() {}

func (*ResourceStatus) ProtoMessage() {}

func (*SELinuxOptions) ProtoMessage() {}

func (*ScaleIOPersistentVolumeSource) ProtoMessage() {}

func (*ScaleIOVolumeSource) ProtoMessage() {}

func (*ScopeSelector) ProtoMessage() {}

func (*ScopedResourceSelectorRequirement) ProtoMessage() {}

func (*SeccompProfile) ProtoMessage() {}

func (*Secret) ProtoMessage() {}

func (*SecretEnvSource) ProtoMessage() {}

func (*SecretKeySelector) ProtoMessage() {}

func (*SecretList) ProtoMessage() {}

func (*SecretProjection) ProtoMessage() {}

func (*SecretReference) ProtoMessage() {}

func (*SecretVolumeSource) ProtoMessage() {}

func (*SecurityContext) ProtoMessage() {}

func (*SerializedReference) ProtoMessage() {}

func (*Service) ProtoMessage() {}

func (*ServiceAccount) ProtoMessage() {}

func (*ServiceAccountList) ProtoMessage() {}

func (*ServiceAccountTokenProjection) ProtoMessage() {}

func (*ServiceList) ProtoMessage() {}

func (*ServicePort) ProtoMessage() {}

func (*ServiceProxyOptions) ProtoMessage() {}

func (*ServiceSpec) ProtoMessage() {}

func (*ServiceStatus) ProtoMessage() {}

func (*SessionAffinityConfig) ProtoMessage() {}

func (*SleepAction) ProtoMessage() {}

func (*StorageOSPersistentVolumeSource) ProtoMessage() {}

func (*StorageOSVolumeSource) ProtoMessage() {}

func (*Sysctl) ProtoMessage() {}

func (*TCPSocketAction) ProtoMessage() {}

func (*Taint) ProtoMessage() {}

func (*Toleration) ProtoMessage() {}

func (*TopologySelectorLabelRequirement) ProtoMessage() {}

func (*TopologySelectorTerm) ProtoMessage() {}

func (*TopologySpreadConstraint) ProtoMessage() {}

func (*TypedLocalObjectReference) ProtoMessage() {}

func (*TypedObjectReference) ProtoMessage() {}

func (*Volume) ProtoMessage() {}

func (*VolumeDevice) ProtoMessage() {}

func (*VolumeMount) ProtoMessage() {}

func (*VolumeMountStatus) ProtoMessage() {}

func (*VolumeNodeAffinity) ProtoMessage() {}

func (*VolumeProjection) ProtoMessage() {}

func (*VolumeResourceRequirements) ProtoMessage() {}

func (*VolumeSource) ProtoMessage() {}

func (*VsphereVirtualDiskVolumeSource) ProtoMessage() {}

func (*WeightedPodAffinityTerm) ProtoMessage() {}

func (*WindowsSecurityContextOptions) ProtoMessage() {}

func (*WorkloadReference) ProtoMessage() {}
