// Copyright 2015 go-swagger maintainers
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package json

import (
	"fmt"
	"reflect"

	"github.com/go-openapi/swag/jsonutils/adapters/ifaces"
)

func Register(dispatcher ifaces.Registrar) {
	t := reflect.TypeOf(Adapter{})
	dispatcher.RegisterFor(
		ifaces.RegistryEntry{
			Who:         fmt.Sprintf("%s.%s", t.PkgPath(), t.Name()),
			What:        ifaces.AllCapabilities,
			Constructor: BorrowAdapterIface,
			Support:     support,
		})
}

func support(_ ifaces.Capability, _ any) bool {
	return true
}
