package types

const (
	// AnnotationSourceType is the annotation key for the renderer type.
	AnnotationSourceType = "manifests.k8s-manifests-lib/source.type"

	// AnnotationSourcePath is the annotation key for the source path/chart identifier.
	AnnotationSourcePath = "manifests.k8s-manifests-lib/source.path"

	// AnnotationSourceFile is the annotation key for the specific template file.
	AnnotationSourceFile = "manifests.k8s-manifests-lib/source.file"
)
