/*
Copyright The Helm Authors.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Package strvals provides tools for working with strval lines.

Helm supports a compressed format for YAML settings which we call strvals.
The format is roughly like this:

	name=value,topname.subname=value

The above is equivalent to the YAML document

	name: value
	topname:
	  subname: value

This package provides a parser and utilities for converting the strvals format
to other formats.
*/
package strvals
